#include "kspread_dlg_data_analysis_template.h"

#include "../dataAnalysis_solver/kdialogeditbutton.h"
#include "../dataAnalysis_solver/qeditbutton.h"

#include "../../kspread_view.h"
#include "../../kspread_sheet.h"
#include "../../selection.h"
#include "../../kspread_map.h"
#include "../../kspread_canvas.h"
#include "../../kspread_doc.h"

#include "../../../kchart/kdchart/KDChartAxisParams.h"
#include "../../../kchart/kchart_part.h"
#include <KDChartTable.h>
#include <KoPartSelectAction.h>

#include <kmessagebox.h>

#define OUTPUT_NAME "Output Range"


KSpreadDlgDataAnalysisTemplate::KSpreadDlgDataAnalysisTemplate(KSpread::View* parent, const char* name, bool modal, const char* caption, int buttonMask) : KDialogEditButton(parent, name, modal, caption, buttonMask)
{
	selectionSheet = NULL;
}

void  KSpreadDlgDataAnalysisTemplate::accept()
{
	KDialogEditButton::accept();
	if (kspreadView->canvasWidget()->activeSheet() != selectionSheet)
			kspreadView->setActiveSheet(selectionSheet);
	kspreadView->selectionInfo()->initialize(QRect(*selectionStartPoint,  *selectionEndPoint), selectionSheet);
	kdDebug() <<"KSpreadDlgDataAnalysisTemplate::accept()" << endl;
	emit accepted();
	delayedDestruct();
}

bool KSpreadDlgDataAnalysisTemplate::checkVectorSizeEquals(QString name, FloatVectorVector& vectorVector)
{
	uint size = 0;
	for (uint i = 0; i<vectorVector.size(); i++)
	{
		if (i > 0)
		{
			if (vectorVector[i].size() != size)
			{
				QString errorStr = "%1 must have the same number of data points.";
				KMessageBox::information(this, errorStr.arg(name));
				return false;
			}
		}
		else
			size = vectorVector[i].size();
	}
	return true;
}

bool KSpreadDlgDataAnalysisTemplate::checkMinVectorSize(QString name, FloatVectorVector& vector, uint minSize)
{
	if (vector.size() >= minSize)
		return true;
	QString errorStr = "Too less rows or columns selected for %1.";
	KMessageBox::information(this, errorStr.arg(name));
	return false;
}

bool KSpreadDlgDataAnalysisTemplate::checkMinVectorSize(QString name, FloatVector& vector, uint minSize)
{
	if (vector.size() >= minSize)
		return true;
	QString errorStr =  name + "%1 must have at least %2 data points.";
	KMessageBox::information(this, errorStr.arg(name).arg(QString::number(minSize)));
	return false;
}

bool KSpreadDlgDataAnalysisTemplate::validateReferences()
{
	for (uint i = 0; i<validationVector.size(); i++)
	{
		RefErrorCode error = KDialogEditButton::validateReference(validationVector[i]);
		QString errorStr = "";
		switch (error)
		{
			case StringIsEmpty:
				errorStr = "%1 missing. Please enter.";
			break;
			case WrongExpression:
				errorStr = "The reference for %1 is not valid, or you did not provide a reference where one was required.\n\n \
				To enter a reference using the mouse, click in the edit box, and then click the cell in the sheet you \
	 			want to refer to, or drag through the range you want to select.";
			break;
			case RefNoError:
				continue;
			break;
		}
		KMessageBox::error(this, errorStr.arg(validationVector[i]->name()), "Error");
		return false;
	}
	return true;
}

SheetOrientation KSpreadDlgDataAnalysisTemplate::detectOrientation(QString name, QRect rect)
{
	if (rect.width() == 1)
		return OrientationColumn;
	if (rect.height() == 1)
		return OrientationRow;
	QString errorStr = "%1 must be a single row or column.";
	KMessageBox::information(this, errorStr.arg(name));
	return OrientationAll;
}


FloatVectorVector* KSpreadDlgDataAnalysisTemplate::getFloats(QString name, KSpread::Sheet* sheet, QRect rect,  SheetOrientation orientation, ValuePolice valuePolice, IntVectorVector* missingVectorVector)
{
	KSpread::Value valueArray = sheet->valueRange(rect.left(), rect.top(), rect.right(), rect.bottom());
	int i_max = 0;
	int j_max = 0;
	FloatVectorVector* vectorVector = new FloatVectorVector();
	switch (orientation)
	{
		case OrientationColumn:
			i_max = rect.width();
			j_max = rect.height();
		break;
		case OrientationRow:
			i_max = rect.height();
			j_max = rect.width();
		break;
		case OrientationAll:
			i_max = rect.width();
			j_max = rect.height();
		break;
	}
	FloatVector* vector = new FloatVector();
	IntVector* missingVector = new IntVector();
	uint count = 0;
	bool nonNumericError = false;
	for (int i = 0; i < i_max; i++)
	{
		for (int j = 0; j < j_max; j++)
		{
			KSpread::Value value = orientation == OrientationRow ? valueArray.element(j, i) : valueArray.element(i, j);
			if (value.isNumber())
				(*vector)[count++] = value.asFloat();
			else
			if (value.isEmpty() ||
			(value.isString() && value.asString() == ""))  // due to KSpread bug?
			{
				if (valuePolice == EmptyShift)
				{
					if (missingVectorVector)
						missingVector->push_back(count);
					count++;
				}
				else
				if (valuePolice == NoEmpty)
					nonNumericError = true;
			}
			else	
				nonNumericError = true;
			if (nonNumericError)
			{
				QString errorStr = "%1 contains non-numeric data.";
				KMessageBox::information(this, errorStr.arg(name));
				delete vector;
				return NULL;
			}
		}
		if (orientation != OrientationAll)
		{
			vectorVector->push_back(*vector);
			if (missingVectorVector)
			{
				missingVectorVector->push_back(*missingVector);
				missingVector = new IntVector();
			}
			vector = new FloatVector();
			count = 0;
		}
	}
	if (orientation == OrientationAll)
		vectorVector->push_back(*vector);
	return vectorVector;
}

QStringVector* KSpreadDlgDataAnalysisTemplate::getLabels(KSpread::Sheet* sheet, QRect& rect, SheetOrientation orientation, bool labels)
{
	int i_max = 0;
	QString rectStr = "Rectangle";
	QStringVector& vector = *(new QStringVector);
	if (!labels)
	{
		QString text;
		if (orientation == OrientationColumn)
		{
			text = "Column";
			i_max = rect.width();
		}
		else
		if (orientation == OrientationRow)
		{
			text = "Row";
			i_max = rect.height();
		}
		else
			return NULL;
		for (int i = 0; i < i_max; i++)
		{
			QString str = text + QString::number(i + 1);
			vector.push_back(str);
		}
	}
	else
	{
		int left = rect.left();
		int top = rect.top();
		int i, j;
		if (orientation == OrientationColumn)
		{
			i = rect.left();
			i_max = rect.right();
			j = top;
			top++;
		}
		else
		if (orientation == OrientationRow)
		{
			i = rect.top();
			i_max = rect.bottom();
			j = left;
			left++;
		}
		else
			return NULL;
		for (; i <= i_max; i++)
		{
			KSpread::Cell* cell = orientation == OrientationRow ? sheet->cellAt(j, i) : sheet->cellAt(i, j);
			vector.push_back(cell->strOutText());
		}
		rect.setTop(top);
		rect.setLeft(left);
	}
	return &vector;
}

KSpread::Sheet* KSpreadDlgDataAnalysisTemplate::createOutput(QWidgetMap& widgetMap, int width, int height,  int& startCol, int& startRow)
{
	enum ErrorCode {NoError, WrongSheetName, SheetNameIsUsed, AreaIsNotEmpty};
	ErrorCode error = NoError;
	KSpread::Sheet* sheet = NULL;
	if (widgetMap["outputGroup"] == "outputRangeButton")
	{
		QString input = widgetMap[OUTPUT_NAME];
		sheet = getSheet(input);
		QRect rect = getRange(input);
		startCol = rect.left();
		startRow = rect.top();
		if (!sheet->areaIsEmpty(QRect(QPoint(startCol, startRow), QSize(width, height))))
			error = AreaIsNotEmpty;
	}
	else
	if (widgetMap["outputGroup"] == "newWorksheetButton")
	{
		QString sheetName = widgetMap["newWorksheetEdit"];
		if (!KSpread::util_validateSheetName(sheetName))
			error = WrongSheetName;
		else
		if (kspreadView->doc()->map()->findSheet(sheetName))
			error = SheetNameIsUsed;
		else
		if ((sheetName.stripWhiteSpace()).isEmpty())
		{
			sheet = kspreadView->doc()->map()->addNewSheet();
			startCol = 1;
			startRow = 1;
		}
		else
		{
			sheet = kspreadView->doc()->map()->addNewSheet();
			sheet->setSheetName(sheetName);
			startCol = 1;
			startRow = 1;
		}
	}
	QString errorStr = "";
	switch (error)
	{
		case WrongSheetName:
			errorStr =  "Output sheet contains illegal characters. Only numbers and letters are allowed.";
		break;
		case SheetNameIsUsed:
			errorStr =  "The name for output sheet is already used.";
		break;
		case AreaIsNotEmpty:
			errorStr = "Output range will overwrite existing data. Press OK to overwrite data.";
			if (KMessageBox::No == KMessageBox::warningYesNo(this, errorStr, QString::null, KStdGuiItem::ok(), KStdGuiItem::cancel()))
				return NULL;
		case NoError:
			selectionSheet = sheet;
			selectionStartPoint = new QPoint(startCol, startRow);
			selectionEndPoint = new QPoint(startCol + width, startRow + height);
			return sheet;
		break;
	}
	KMessageBox::information( this, errorStr);
	return NULL;
}

QPen KSpreadDlgDataAnalysisTemplate::changeCellLineStyle(QPen currentPen, LineStyle style)
{
	switch (style)
	{
		case NoLine:
			currentPen.setStyle(NoPen);
		break;
		case ThinLine:
			currentPen.setStyle(SolidLine);
			currentPen.setWidth(0);
		break;
		case FatLine:
			currentPen.setStyle(SolidLine);
			currentPen.setWidth(2);
		break;
	}
	return currentPen;
}

void KSpreadDlgDataAnalysisTemplate::setCellAlign(KSpread::Cell* cell, CellAlign align)
{
	KSpread::Format* format = cell->format();
	switch(align)
	{
		case StandardAlign:
			format->setAlign(KSpread::Format::Undefined);
		break;
		case CenterAlign:
			format->setAlign(KSpread::Format::Center);
		break;
	}
}

void KSpreadDlgDataAnalysisTemplate::setCellStyle(KSpread::Sheet* sheet, int col, int row, LineStyle topLine, LineStyle bottomLine, FontType font, CellAlign align)
{
	KSpread::Cell* cell = sheet->nonDefaultCell(col, row);
	KSpread::Format* format = cell->format();
	QPen top = format->topBorderStyle(col, row);
	QPen bottom = format->bottomBorderStyle(col, row);
//	if (cell->isObscuringForced())
//		cell = cell->obscuringCells().first();
	format->setTopBorderPen(changeCellLineStyle(top, topLine));
	format->setBottomBorderPen(changeCellLineStyle(bottom, bottomLine));
	setCellAlign(cell, align);
	format->setTextFontItalic(font == FontItalic);
}

void KSpreadDlgDataAnalysisTemplate::setCell(KSpread::Sheet* sheet, int col, int row, QString text, LineStyle topLine, LineStyle bottomLine, FontType font, CellAlign align)
{
	sheet->setText(row, col, text);
	setCellStyle(sheet, col, row, topLine, bottomLine, font, align);
}

void KSpreadDlgDataAnalysisTemplate::setTitlesRow(KSpread::Sheet* sheet, int colStart, int colEnd,  int row)
{
	for (int i = colStart; i <= colEnd; i++)
		setCellStyle(sheet, i, row, FatLine, ThinLine, FontItalic, CenterAlign);
}


void KSpreadDlgDataAnalysisTemplate::setEndRow(KSpread::Sheet* sheet, int colStart, int colEnd,  int row)
{
	for (int i = colStart; i <= colEnd; i++)
		setCellStyle(sheet, i, row, NoLine, FatLine);

}

int KSpreadDlgDataAnalysisTemplate::getScreenWidth(KSpread::Sheet* sheet, int cell1, int  cell2)
{
	if (cell1 > cell2)
		return sheet->columnPos(cell1) - sheet->columnPos(cell2);
	else
		return sheet->columnPos(cell2) - sheet->columnPos(cell1);
}

int KSpreadDlgDataAnalysisTemplate::getScreenHeight(KSpread::Sheet* sheet, int cell1, int  cell2)
{
	if (cell1 > cell2)
		return sheet->rowPos(cell1 + 1) - sheet->rowPos(cell2 + 1);
	else
		return sheet->rowPos(cell2 + 1) - sheet->rowPos(cell1 + 1);
}

void KSpreadDlgDataAnalysisTemplate::insertChart(ChartParameters* chartParams)
{

	KSpread::Sheet* sheet = selectionSheet;

	int posX = getScreenWidth(sheet, 0, chartParams->startPoint.x());
	int posY = getScreenHeight(sheet, 0, chartParams->startPoint.y());
	int width = getScreenWidth(sheet, chartParams->startPoint.x(), chartParams->startPoint.x() + chartParams->size.width());
	int height = getScreenHeight(sheet, chartParams->startPoint.y(), chartParams->startPoint.y() + chartParams->size.height());
	KoRect kRect(posX, posY, width, height);

	QValueList<KoDocumentEntry> vec = KoDocumentEntry::query(true, "'KOfficeChart' in ServiceTypes");
	if (vec.isEmpty())
	{
		kdDebug() << "error in getting KOfficeChart" << endl;
		return;
	}
	KoDocumentEntry e = vec[0];
	kdDebug(36001) << "Creating document" << endl;
	KoDocument* d = e.createDoc(sheet->doc());
	kdDebug(36001) << "Created" << endl;
	if (!d)
	{
		kdDebug() << "Error inserting child!" << endl;
		return;
	}

	if ( !d->initDoc(KoDocument::InitDocEmbedded) )
		return;

	KSpread::EmbeddedKOfficeObject* ch = new KSpread::EmbeddedKOfficeObject(sheet->doc(), sheet, d, kRect);
	KChart::KChartPart* chartDoc = (KChart::KChartPart*) ch->embeddedObject()->document();
	FloatVectorVector& values = *chartParams->xValues;
	QStringVector& xLabels = *chartParams->xLabels;
	QStringVector& yLabels = *chartParams->yLabels;
	KChart::KChartParams* params = chartDoc->params();
	KDChartTableData* matrix = chartDoc->data();

	params->setDataColor(0, QColor("red"));
	params->setDataColor(1, QColor("blue"));

	switch (chartParams->chartType)
	{
		case ChartBar:
			params->setChartType(KChart::KChartParams::Bar);
			params->setBarChartSubType(KChart::KChartParams::BarNormal);
			params->setThreeDBars(false);
		break;
		case ChartLine:
			params->setChartType(KChart::KChartParams::Line);
			params->setLineChartSubType(KChart::KChartParams::LineNormal);
			KDChartAxisParams aParams;
			aParams = params->axisParams(KDChartAxisParams::AxisPosLeft);
			aParams.setAxisCalcMode(KDChartAxisParams::AxisCalcLinear);
			params->setAxisParams(KDChartAxisParams::AxisPosLeft, aParams);
			params->setLineMarker(true);
		break;
	}
	int fontSize;
	QFont font = params->header1Font();
	fontSize = int (params->headerFooterFontRelSize(KDChartParams::HdFtPosHeader) * 2);

	bool useRelSize = params->headerFooterFontUseRelSize(KDChartParams::HdFtPosHeader);
	params->setHeaderFooterFont(KDChartParams::HdFtPosHeader, font, useRelSize, fontSize);

	fontSize = int (params->axisTitleFontRelSize(KDChartAxisParams::AxisPosBottom) * 1.5);
	params->setAxisTitleFontRelSize(KDChartAxisParams::AxisPosBottom, fontSize);

	fontSize = int (params->axisTitleFontRelSize(KDChartAxisParams::AxisPosLeft) * 1.5);
	params->setAxisTitleFontRelSize(KDChartAxisParams::AxisPosLeft, fontSize);

	fontSize = int (params->legendFontRelSize() * 2);
	params->setLegendFontRelSize(fontSize);

	fontSize = int (params->legendTitleFontRelSize() * 1.7);
	params->setLegendTitleFontRelSize(fontSize);

	params->setAxisTitle(KDChartAxisParams::AxisPosBottom, chartParams->xAxisName);
	params->setAxisTitle(KDChartAxisParams::AxisPosLeft, chartParams->yAxisName);

	params->setHeader1Text(chartParams->chartName);
	params->setFooterText(" ");

	matrix->setUsedRows(xLabels.size());
	matrix->setUsedCols(yLabels.size());
	matrix->expand(xLabels.size(), yLabels.size());
	for (uint i = 0; i < yLabels.size(); i++)
	{
		FloatVector& valuesRow = values[i];
		for (uint j = 0; j < xLabels.size(); j++)
			if (valuesRow.contains(j))
				matrix->setCell(j,  i, KoChart::Value(valuesRow[j]));
	}

	QStringList& colLabels = chartDoc->colLabelTexts();
	QStringList& rowLabels = chartDoc->rowLabelTexts();

	rowLabels.clear();
	for(uint row = 0; row < xLabels.size(); row++)
		rowLabels << xLabels[row];

	colLabels.clear();
	for(uint col = 0; col < yLabels.size(); col++)
		colLabels << yLabels[col];

	chartDoc->setModified(true);
	sheet->doc()->insertObject(ch);
//	sheet->updateView(ch->boundingRect());
}

#include "kspread_dlg_data_analysis_template.moc"
